﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  public abstract class ShoveExpression<T> : StatelessExpression
  {
    public void Eval(IStack<T> stack, IStack<long> integerStack, int count = 1)
    {
      if (integerStack.Count == 0) return;

      if ((integerStack.Top > stack.Count - count) || (integerStack.Top < 0) || (stack.Count < 2)) return;

      var index = (int)integerStack.Pop();
      var item = stack.Pop();

      stack.Insert(index, item);
    }
  }

  public class IntegerShoveExpression : ShoveExpression<long>
  {
    protected override string InitStringRepresentation()
    {
      return "INTEGER.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
    }
  }

  public class FloatShoveExpression : ShoveExpression<double>
  {
    protected override string InitStringRepresentation()
    {
      return "FLOAT.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.FloatStack, interpreter.IntegerStack);
    }
  }

  public class BooleanShoveExpression : ShoveExpression<bool>
  {
    protected override string InitStringRepresentation()
    {
      return "BOOLEAN.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.BooleanStack, interpreter.IntegerStack);
    }
  }

  public class NameShoveExpression : ShoveExpression<string>
  {
    protected override string InitStringRepresentation()
    {
      return "NAME.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.NameStack, interpreter.IntegerStack);
    }
  }

  public class ExecShoveExpression : ShoveExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "EXEC.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.ExecStack, interpreter.IntegerStack);
    }
  }

  public class CodeShoveExpression : ShoveExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "CODE.SHOVE";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.CodeStack, interpreter.IntegerStack);
    }
  }
}