﻿using System;

namespace HeuristicLab.Problems.PushGP.ProgramSynthesis {
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  [Creatable(CreatableAttribute.Categories.GeneticProgrammingProblems, Priority = 180)]
  [Item("Push GP Problem", "")]
  public class PushProblem : SymbolicExpressionTreeProblem {

    private readonly PushGpInterpreterPool pool = new PushGpInterpreterPool(new Configuration());

    // persistence
    [StorableConstructor]
    protected PushProblem(bool deserializing) : base(deserializing) { }

    // cloning
    protected PushProblem(PushProblem original, Cloner cloner)
      : base(original, cloner) {
    }

    protected PushProblem() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      throw new NotImplementedException();
    }

    public Configuration Configuration { get { return this.pool.Configuration; } }

    public override bool Maximization { get { return false; } }

    public override double Evaluate(ISymbolicExpressionTree tree, IRandom random) {
      var program = tree as PushProgram;

      using (var interpreter = this.pool.GetInstance()) {
        interpreter.Interpret(program);
      }

      return 0d;
    }
  }
}
